const TTP = require('@tabletop-playground/api');



class PlayerPickerDie {
    constructor(dieObject){
        this.dieObject = dieObject;
        this.faceColorsToDisplay = [];
        this.faceMarked = [
            {//0 — Undefined
                rot: [0,0,0],
                pos: [0,0,0]
            },
            {
                rot: [57,89,55],
                pos: [-0.05,-0.75,0.48]
            },
            {
                rot: [-31,0,73],
                pos: [0.45,0,0.77]
            },
            {
                rot: [90,148,163],
                pos: [0.77,-0.45,0]
            },
            {
                rot: [-150,0,45],
                pos: [0.52,0,-0.73]
            },
            {
                rot: [90,32,0],
                pos: [-0.77,-0.45,-0]
            },
            {
                rot: [-60,90,50],
                pos: [-0.0,0.76,0.47]
            },
            {
                rot: [-120,-90,0],
                pos: [-0.0,-0.76,-0.47]
            },
            {
                rot: [-90,32,-35],
                pos: [0.76,0.47,-0.0]
            },
            {
                rot: [31,0,-0],
                pos: [-0.43,0.0,0.77]
            },
            {
                rot: [91,-34,70],
                pos: [-0.76,0.48,0]
            },
            {
                rot: [148,-0,0],
                pos: [-0.46,0,-0.75]
            },
            {
                rot: [120,-90,0],
                pos: [-0.0,0.80,-0.42]
            }
        ]

        this.resetDie();
        TTP.globalEvents.onPlayerSwitchedSlots.add(()=>this.resetDie());
        TTP.globalEvents.onPlayerJoined.add(()=>this.resetDie());
        TTP.globalEvents.onPlayerLeft.add(()=>this.resetDie());
    }

    resetDie(){
        this.setFacesToPlayers();
        this.generateUIFaces();
    }

    setColor(index, color){

        this.faceMarked[index].uiElement.widget.setColor(color);
    }

    removeColor(index, color){

    }

    generateUIFaces(){
        let perPlayer = Math.floor(12 / this.faceColorsToDisplay.length);
        for (let goUI of this.dieObject.getUIs()){
            this.dieObject.removeUIElement(goUI);
        }
        
        if (perPlayer > 0){
            let facesToGenerate = this.faceColorsToDisplay.flatMap(i => Array(perPlayer).fill(i));
            let i = 1;
            for (let colorFace of facesToGenerate){
                let markedFace = this.faceMarked[i++];

                let ui = new TTP.UIElement();
                ui.useWidgetSize = false;
                ui.height = 7.5;
                ui.width = 7.5;
                ui.scale = 1;
                ui.rotation = markedFace.rot;
                ui.position = markedFace.pos;
                let w = new TTP.Border();
                w.setColor(colorFace);
                ui.widget = w;
                this.dieObject.addUI(ui);
    
                markedFace.uiElement = ui;
            }
        }
        /*
        for (let markedFace of this.faceMarked){
            let ui = new TTP.UIElement();
            ui.useWidgetSize = false;
            ui.height = 7.5;
            ui.width = 7.5;
            ui.scale = 1;
            ui.rotation = markedFace.rot;
            ui.position = markedFace.pos;
            let w = new TTP.Border();
            //w.setColor([5,5,5]);
            //w.setColor([255,245,186]);
            //w.setEnabled(false);
            ui.widget = w;
            this.dieObject.addUI(ui);

            markedFace.uiElement = ui;
        }*/ 
        
    }
    setFacesToPlayers(){
        let players = TTP.world.getAllPlayers();
        this.faceColorsToDisplay = [];
        for (let player of players) {
            this.faceColorsToDisplay.push(player.getPlayerColor());
        }
    }
}

TTP.refDice.onCreated.add((dieObject)=>{
    new PlayerPickerDie(dieObject);
});